/**************************************************************************
  This is an example for our Monochrome OLEDs based on SSD1306 drivers

  Pick one up today in the adafruit shop!
  ------> http://www.adafruit.com/category/63_98

  This example is for a 128x32 pixel display using SPI to communicate
  4 or 5 pins are required to interface.

  Adafruit invests time and resources providing this open
  source code, please support Adafruit and open-source
  hardware by purchasing products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries,
  with contributions from the open source community.
  BSD license, check license.txt for more information
  All text above, and the splash screen below must be
  included in any redistribution.
 **************************************************************************/

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <SSD1306_FlexIO.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels

#define OLED_DC    21
#define OLED_CS    20
#define OLED_RESET 15
SSD1306_FlexIO_32 display1(OLED_DC, OLED_RESET, OLED_CS, &SPI);

// Declaration for SSD1306 display connected using software SPI (default case):
#define OLED_DC2    5
#define OLED_CS2    6
#define OLED_RESET2 7
FlexSPI SPIFLEX(2, 3, 4, -1); // Setup on (int mosiPin, int sckPin, int misoPin, int csPin=-1) :
SSD1306_FlexIO_32 display2(OLED_DC2, OLED_RESET2, OLED_CS2, &SPIFLEX);

/* Comment out above, uncomment this block to use hardware SPI
  #define OLED_DC     6
  #define OLED_CS     7
  #define OLED_RESET  8
  SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT,
  &SPI, OLED_DC, OLED_RESET, OLED_CS);
*/

#define NUMFLAKES     10 // Number of snowflakes in the animation example

#define LOGO_HEIGHT   16
#define LOGO_WIDTH    16
static const unsigned char PROGMEM logo_bmp[] =
{ B00000000, B11000000,
  B00000001, B11000000,
  B00000001, B11000000,
  B00000011, B11100000,
  B11110011, B11100000,
  B11111110, B11111000,
  B01111110, B11111111,
  B00110011, B10011111,
  B00011111, B11111100,
  B00001101, B01110000,
  B00011011, B10100000,
  B00111111, B11100000,
  B00111111, B11110000,
  B01111100, B11110000,
  B01110000, B01110000,
  B00000000, B00110000
};

void setup() {
  Serial.begin(9600);

  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if (!display1.begin(SSD1306_SWITCHCAPVCC)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;); // Don't proceed, loop forever
  }
  if (!display2.begin(SSD1306_SWITCHCAPVCC)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;); // Don't proceed, loop forever
  }

  // Show initial display buffer contents on the screen --
  // the library initializes this with an Adafruit splash screen.
  display1.display();
  display2.display();
  delay(2000); // Pause for 2 seconds

  // Clear the buffer
  display1.clearDisplay();
  display2.clearDisplay();

  // Draw a single pixel in white
  display1.drawPixel(10, 10, WHITE);
  display2.drawPixel(10, 10, WHITE);

  // Show the display buffer on the screen. You MUST call display() after
  // drawing commands to make them visible on screen!
  display1.display();
  display2.display();
  delay(2000);
  // display1.display() is NOT necessary after every single drawing command,
  // unless that's what you want...rather, you can batch up a bunch of
  // drawing operations and then update the screen all at once by calling
  // display1.display(). These examples demonstrate both approaches...

  testdrawline(display1);      // Draw many lines
  testdrawline(display2);      // Draw many lines

  testdrawrect(display1);      // Draw rectangles (outlines)
  testdrawrect(display2);      // Draw rectangles (outlines)

  testfillrect(display1);      // Draw rectangles (filled)
  testfillrect(display2);      // Draw rectangles (filled)

  testdrawcircle(display1);    // Draw circles (outlines)
  testdrawcircle(display2);    // Draw circles (outlines)

  testfillcircle(display1);    // Draw circles (filled)

  testdrawroundrect(display1); // Draw rounded rectangles (outlines)

  testfillroundrect(display1); // Draw rounded rectangles (filled)

  testdrawtriangle(display1);  // Draw triangles (outlines)

  testfilltriangle(display1);  // Draw triangles (filled)

  testdrawchar(display1);      // Draw characters of the default font

  testdrawstyles(display1);    // Draw 'stylized' characters

  testscrolltext(display1);    // Draw scrolling text

  testdrawbitmap(display1);    // Draw a small bitmap image

  // Invert and restore display, pausing in-between
  display1.invertDisplay(true);
  delay(1000);
  display1.invertDisplay(false);
  delay(1000);

  testanimate(logo_bmp, LOGO_WIDTH, LOGO_HEIGHT); // Animate bitmaps
}

void loop() {
}

void testdrawline(SSD1306_FlexIO &disp) {
  int16_t i;

  disp.clearDisplay(); // Clear display buffer

  for (i = 0; i < disp.width(); i += 4) {
    disp.drawLine(0, 0, i, disp.height() - 1, WHITE);
    disp.display(); // Update screen with each newly-drawn line
    delay(1);
  }
  for (i = 0; i < disp.height(); i += 4) {
    disp.drawLine(0, 0, disp.width() - 1, i, WHITE);
    disp.display();
    delay(1);
  }
  delay(250);

  disp.clearDisplay();

  for (i = 0; i < disp.width(); i += 4) {
    disp.drawLine(0, disp.height() - 1, i, 0, WHITE);
    disp.display();
    delay(1);
  }
  for (i = disp.height() - 1; i >= 0; i -= 4) {
    disp.drawLine(0, disp.height() - 1, disp.width() - 1, i, WHITE);
    disp.display();
    delay(1);
  }
  delay(250);

  disp.clearDisplay();

  for (i = disp.width() - 1; i >= 0; i -= 4) {
    disp.drawLine(disp.width() - 1, disp.height() - 1, i, 0, WHITE);
    disp.display();
    delay(1);
  }
  for (i = disp.height() - 1; i >= 0; i -= 4) {
    disp.drawLine(disp.width() - 1, disp.height() - 1, 0, i, WHITE);
    disp.display();
    delay(1);
  }
  delay(250);

  disp.clearDisplay();

  for (i = 0; i < disp.height(); i += 4) {
    disp.drawLine(disp.width() - 1, 0, 0, i, WHITE);
    disp.display();
    delay(1);
  }
  for (i = 0; i < disp.width(); i += 4) {
    disp.drawLine(disp.width() - 1, 0, i, disp.height() - 1, WHITE);
    disp.display();
    delay(1);
  }

  delay(2000); // Pause for 2 seconds
}

void testdrawrect(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  for (int16_t i = 0; i < disp.height() / 2; i += 2) {
    disp.drawRect(i, i, disp.width() - 2 * i, disp.height() - 2 * i, WHITE);
    disp.display(); // Update screen with each newly-drawn rectangle
    delay(1);
  }

  delay(2000);
}

void testfillrect(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  for (int16_t i = 0; i < disp.height() / 2; i += 3) {
    // The INVERSE color is used so rectangles alternate white/black
    disp.fillRect(i, i, disp.width() - i * 2, disp.height() - i * 2, INVERSE);
    disp.display(); // Update screen with each newly-drawn rectangle
    delay(1);
  }

  delay(2000);
}

void testdrawcircle(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  for (int16_t i = 0; i < max(disp.width(), disp.height()) / 2; i += 2) {
    disp.drawCircle(disp.width() / 2, disp.height() / 2, i, WHITE);
    disp.display();
    delay(1);
  }

  delay(2000);
}

void testfillcircle(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  for (int16_t i = max(disp.width(), disp.height()) / 2; i > 0; i -= 3) {
    // The INVERSE color is used so circles alternate white/black
    disp.fillCircle(disp.width() / 2, disp.height() / 2, i, INVERSE);
    disp.display(); // Update screen with each newly-drawn circle
    delay(1);
  }

  delay(2000);
}

void testdrawroundrect(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  for (int16_t i = 0; i < disp.height() / 2 - 2; i += 2) {
    disp.drawRoundRect(i, i, disp.width() - 2 * i, disp.height() - 2 * i,
                       disp.height() / 4, WHITE);
    disp.display();
    delay(1);
  }

  delay(2000);
}

void testfillroundrect(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  for (int16_t i = 0; i < disp.height() / 2 - 2; i += 2) {
    // The INVERSE color is used so round-rects alternate white/black
    disp.fillRoundRect(i, i, disp.width() - 2 * i, disp.height() - 2 * i,
                       disp.height() / 4, INVERSE);
    disp.display();
    delay(1);
  }

  delay(2000);
}

void testdrawtriangle(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  for (int16_t i = 0; i < max(disp.width(), disp.height()) / 2; i += 5) {
    disp.drawTriangle(
      disp.width() / 2  , disp.height() / 2 - i,
      disp.width() / 2 - i, disp.height() / 2 + i,
      disp.width() / 2 + i, disp.height() / 2 + i, WHITE);
    disp.display();
    delay(1);
  }

  delay(2000);
}

void testfilltriangle(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  for (int16_t i = max(disp.width(), disp.height()) / 2; i > 0; i -= 5) {
    // The INVERSE color is used so triangles alternate white/black
    disp.fillTriangle(
      disp.width() / 2  , disp.height() / 2 - i,
      disp.width() / 2 - i, disp.height() / 2 + i,
      disp.width() / 2 + i, disp.height() / 2 + i, INVERSE);
    disp.display();
    delay(1);
  }

  delay(2000);
}

void testdrawchar(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  disp.setTextSize(1);      // Normal 1:1 pixel scale
  disp.setTextColor(WHITE); // Draw white text
  disp.setCursor(0, 0);     // Start at top-left corner
  disp.cp437(true);         // Use full 256 char 'Code Page 437' font

  // Not all the characters will fit on the disp. This is normal.
  // Library will draw what it can and the rest will be clipped.
  for (int16_t i = 0; i < 256; i++) {
    if (i == '\n') disp.write(' ');
    else          disp.write(i);
  }

  disp.display();
  delay(2000);
}

void testdrawstyles(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  disp.setTextSize(1);             // Normal 1:1 pixel scale
  disp.setTextColor(WHITE);        // Draw white text
  disp.setCursor(0, 0);            // Start at top-left corner
  disp.println(F("Hello, world!"));

  disp.setTextColor(BLACK, WHITE); // Draw 'inverse' text
  disp.println(3.141592);

  disp.setTextSize(2);             // Draw 2X-scale text
  disp.setTextColor(WHITE);
  disp.print(F("0x")); disp.println(0xDEADBEEF, HEX);

  disp.display();
  delay(2000);
}

void testscrolltext(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  disp.setTextSize(2); // Draw 2X-scale text
  disp.setTextColor(WHITE);
  disp.setCursor(10, 0);
  disp.println(F("scroll"));
  disp.display();      // Show initial text
  delay(100);

  // Scroll in various directions, pausing in-between:
  disp.startscrollright(0x00, 0x0F);
  delay(2000);
  disp.stopscroll();
  delay(1000);
  disp.startscrollleft(0x00, 0x0F);
  delay(2000);
  disp.stopscroll();
  delay(1000);
  disp.startscrolldiagright(0x00, 0x07);
  delay(2000);
  disp.startscrolldiagleft(0x00, 0x07);
  delay(2000);
  disp.stopscroll();
  delay(1000);
}

void testdrawbitmap(SSD1306_FlexIO &disp) {
  disp.clearDisplay();

  disp.drawBitmap(
    (disp.width()  - LOGO_WIDTH ) / 2,
    (disp.height() - LOGO_HEIGHT) / 2,
    logo_bmp, LOGO_WIDTH, LOGO_HEIGHT, 1);
  disp.display();
  delay(1000);
}

#define XPOS   0 // Indexes into the 'icons' array in function below
#define YPOS   1
#define DELTAY 2

void testanimate(const uint8_t *bitmap, uint8_t w, uint8_t h) {
  int8_t f, icons[NUMFLAKES][3];

  // Initialize 'snowflake' positions
  for (f = 0; f < NUMFLAKES; f++) {
    icons[f][XPOS]   = random(1 - LOGO_WIDTH, display1.width());
    icons[f][YPOS]   = -LOGO_HEIGHT;
    icons[f][DELTAY] = random(1, 6);
    Serial.print(F("x: "));
    Serial.print(icons[f][XPOS], DEC);
    Serial.print(F(" y: "));
    Serial.print(icons[f][YPOS], DEC);
    Serial.print(F(" dy: "));
    Serial.println(icons[f][DELTAY], DEC);
  }

  for (;;) { // Loop forever...
    display1.clearDisplay(); // Clear the display buffer
    display2.clearDisplay(); // Clear the display buffer

    // Draw each snowflake:
    for (f = 0; f < NUMFLAKES; f++) {
      display1.drawBitmap(icons[f][XPOS], icons[f][YPOS], bitmap, w, h, WHITE);
      display2.drawBitmap(icons[f][XPOS], icons[f][YPOS], bitmap, w, h, WHITE);
    }

    display1.displayAsync(); // Show the display buffer on the screen
    display2.display(); // Show the display buffer on the screen
    delay(100);        // Pause for 1/10 second

    // Then update coordinates of each flake...
    for (f = 0; f < NUMFLAKES; f++) {
      icons[f][YPOS] += icons[f][DELTAY];
      // If snowflake is off the bottom of the screen...
      if (icons[f][YPOS] >= display1.height()) {
        // Reinitialize to a random position, just off the top
        icons[f][XPOS]   = random(1 - LOGO_WIDTH, display1.width());
        icons[f][YPOS]   = -LOGO_HEIGHT;
        icons[f][DELTAY] = random(1, 6);
      }
    }
  }
}
